(function(root, factory) {
    root.returnExports = {
        fixedDownloadForGmBox: factory(root.jQuery)
    };
}(this, function($) {
    var Sequence = function(_opts) {
        var __self = this;
        this.timer = null;
        this.tmpImg = new Image();

        // 获取id和该dom元素
        this.id = _opts.id; // 必须值，值为目标id
        this.target = document.getElementById(this.id); // 获取该对象

        // 获取标签设置属性值
        this.src = this.target.getAttribute('data-src'); // 必须值，值为目标的背景图地址
        this.frames = parseInt(this.target.getAttribute('data-frames'), 10); // 必须值，值为目标的帧数
        this.isLoop = eval(this.target.getAttribute('data-loop')); // 必须值，默认值为true，即无限循环播放
        this.looptime = parseInt(this.target.getAttribute('data-looptime'), 10) || 200; // 非必须，为播放速率，默认值为200

        // 键名检查
        this._checkKeys({
            id: this.id,
            src: this.src,
            frames: this.frames
        });
        // 键值类型检查
        /*this._checkValues({
          id: this.id,
          src: this.src,
          frames: this.frames
        });*/

        // 动态获取图片宽高
        this.tmpImg.src = this.src;
        this.tmpImg.onload = function() {
            __self.width = __self.tmpImg.width;
            __self.height = __self.tmpImg.height / __self.frames;

            // 设置目标形状和背景图
            __self._setSharpAndBg();
        }
    };

    /**
     * 键名检查
     * @param {object} _opts - the object need to check.
     * @return {boolean} The result of the check.
     */
    Sequence.prototype._checkKeys = function(_opts) {
        var isCheck = true;
        var keys = ['id', 'src', 'frames'];
        for (var i = 0; i < keys.length; i++) {
            if (!_opts.hasOwnProperty(keys[i]) && i < keys.length) {
                isCheck = false;
                alert('Sequence初始化参数有误，请重新检查参数。')
                return false;
            }
        }
    };

    /**
     * 键值检查
     * @param {object} _opts - the object need to check.
     * @return {boolean} The result of the check.
     */
    Sequence.prototype._checkValues = function(_opts) {
        var isCheck = true;
        var types = ['string', 'string', 'number'];
        var idx = 0;
        for (var key in _opts) {
            if (typeof _opts[key] !== types[idx] && idx < types.length) {
                isCheck = false;
                alert('Sequence初始化参数值的类型有误，请重新检查参数。')
                return false;
            }
            idx++;
        }
    };

    /**
     * 设置目标形状和背景图
     * @return {boolean} The result of the set.
     */
    Sequence.prototype._setSharpAndBg = function() {
        this.target.style.width = this.width;
        this.target.style.height = this.height;
        this.target.style.backgroundImage = 'url("' + this.src + '")';
        this.target.style.backgroundRepeat = 'no-repeat';
        this.target.style.backgroundPosition = '0px 0px';
        return true;
    };

    /**
     * 启动序列图动画
     * @return {boolean} The result of the start.
     */
    Sequence.prototype.start = function() {
        var that = this;
        if (that.timer) return;
        this.currframe = 1;
        that.timer = setInterval(function() {
            if (that.currframe < that.frames) {
                that.target.style.backgroundPositionY = -(that.currframe * (parseInt(that.height, 10)+1)) + 'px';
                that.currframe++;
            } else {
                that.currframe = 1;

                // 只循环1次
                if (!that.isLoop) {
                    that.stop();
                }
            }
        }, that.looptime);
        return true;
    };

    /**
     * 关闭序列图动画
     * @return {boolean} The result of the stop.
     */
    Sequence.prototype.stop = function() {
        clearInterval(this.timer);
        return true;
    };

    var ImageLoader = {
        imgList: [],
        loader: function(path, callback) {
            var img = new Image();
            img.onload = function() {
                img.onload = null;
                callback(path);
            }
            img.src = path;
        },
        bootstrap: function(config) {
            ImageLoader.imgList = config.imgList || [];

            var len = ImageLoader.imgList.length,
                i = 0;
            while (ImageLoader.imgList.length) {
                ImageLoader.loader(ImageLoader.imgList.shift(), function(path) {
                    config.done(path, ++i, len);
                });
            }
        }
    };

    var fixedDownloadForGmBox = {
    	gifObj1: null,
    	gifObj2: null,
        // 弹窗id
        tipsId: 'J_tips_gmbox_download',
        // 弹窗关闭按钮id
        closeBtId: 'J_tips_gmbox_download_bt',
        enterGmboxGifId: 'enter-sequence',
        loopGmboxGifId: 'loop-sequence',
        // 下载地址
        downloadUrl: '//www.gm99.com/gamebox',
        // 弹窗样式
        css: '<style type="text/css">\
	        .tips-gmbox-download{\
	            position:fixed;left:0;bottom:0;width:315px;height:310px;z-index:99999;\
	        }\
	        .tips-gmbox-download h1{\
	            position:absolute;left:-70px;bottom:-60px;width:409px;height:408px;cursor:pointer;transform:scale(0.8);\
	        }\
	        .loop-sequence{display:none;}\
	        .tips-gmbox-download-bt{display:none;width:26px;height:26px;position:absolute;left:21px;bottom:270px;background-image:url(http://www.gmresstatic.com/img/gmbox/gmbox-close-btn.png);z-index:999;}\
	        </style>',
        // 弹窗结构 http://www.gmresstatic.com/img/gmbox/gmbox-enter.gif
        html: '<div class="tips-gmbox-download" id="J_tips_gmbox_download">\
	            <a href="javascript:;" class="tips-gmbox-download-bt" id="J_tips_gmbox_download_bt"></a>\
	            <h1 id="enter-sequence" class="enter-sequence" data-src="//www.gmresstatic.com/img/gmbox/gmbox-enter.png" data-frames="100" data-looptime="33" data-loop="false"></h1><h1 id="loop-sequence" class="loop-sequence" data-src="//www.gmresstatic.com/img/gmbox/gmbox-loop.png" data-frames="100" data-looptime="33" data-loop="true"></h1>\
	        </div>',
        // 阻止冒泡
        stopBubble: function(e) {
            if (e && e.stopPropagation) {
                e.stopPropagation();
            } else {
                window.event.cancelBubble = true;
            }
        },
        // 向文档插入样式\结构
        append2doc: function(str, tag) {
            var temp = document.createElement('div');
            temp.innerHTML = 'x' + str;
            tag = document.getElementsByTagName(tag)[0];
            tag.appendChild(temp.lastChild);
        },
        // 创建css和dom
        createDownloadPop: function() {
            var self = this;
            ImageLoader.bootstrap({
                imgList: [
                    '//www.gmresstatic.com/img/gmbox/gmbox-enter.png',
                    '//www.gmresstatic.com/img/gmbox/gmbox-loop.png'
                ],
                done: function(path, idx, len) {
                    if (idx === len) {
                        // 插入样式
                        self.append2doc(self.css, 'head');
                        // 插入dom
                        self.append2doc(self.html, 'body');
                        self.gifObj1 = new Sequence({
                            id: self.enterGmboxGifId
                        });
                        self.gifObj1.start();
                        setTimeout(function() {
                            self.gifObj2 = new Sequence({
                                id: self.loopGmboxGifId
                            });
                            self.gifObj2.start();
                            document.getElementById(self.enterGmboxGifId).style.display = 'none';
                            document.getElementById(self.loopGmboxGifId).style.display = 'block';
                            document.getElementById(self.closeBtId).style.display = 'block';
                            self.initEventHandler();
                            // '下载图标展示'统计
                            self.statistics('show');
                        }, 3200);
                    }
                }
            });

        },
        // 绑定事件
        initEventHandler: function() {
            var self = this;
            // 弹窗
            var tips = window[self.tipsId];
            // 弹窗关闭按钮
            var bt = window[self.closeBtId];
            // 跳转到盒子下载页
            tips.onclick = function() {
                window.open(self.downloadUrl);
                // '下载图标点击'统计
                self.statistics('downloadClick');
            }
            // 关闭盒子下载弹窗
            bt.onclick = function(e) {
            	self.gifObj1.stop();
            	self.gifObj2.stop();
                tips.style.display = 'none';
                self.stopBubble(e);
                // '下载图标关闭'统计
                self.statistics('downloadClose');
            }
        },
        // 是否平台页面判断
        isPlatform: function() {
            if (window.location.host == 'www.gm99.com') {
                return true;
            } else {
                return false;
            }
        },
        // 是否游戏页面判断
        isGamePage: function() {
            var state = window.location.href.indexOf('/play_games/play/server/');
            if (state != -1) {
                return true;
            } else {
                return false;
            }
        },
        // 判断用户是否是盒子用户
        boxUser: function(install, uninstall) {
            // 非平台，游戏，微端均不显示
            var self = this;
            if (!self.isPlatform() || self.isGamePage() || self.isLauncher() || !$) { return; }
            $.getJSON('//www.gm99.com/gmbox/is_install?need_token=0', function(json) {
                if (json.result == 1) {
                    if (!json.data.install) {
                        install && install();
                    } else {
                        uninstall && uninstall();
                    }
                    // 未登录时显示弹窗
                } else if (json.result == -1) {
                    install && install();
                } else {
                    uninstall && uninstall();
                }
            });
        },

        // 关闭右侧gmbox下载入口
        hideRightDownloadBt: function() {
            $('.rfDown').remove();
            $('#rightFloat .rfDownP .rfGmIcon').css('top', '-8px');
            $('.rfFbIcon').css('position', 'relative');
        },

        // 是否微端
        isLauncher: function() {
            var state = window.location.href.indexOf('/launcher');
            return state != -1 ? true : false;
        },
        // 统计代码
        statistics: function(type) {
            var self = this;
            var statistics = {
                show: 'gm-hz-pt-icon-show|下载图标展示',
                downloadClick: 'gm-hz-pt-icon-click|下载图标点击',
                downloadClose: 'gm-hz-pt-icon-close|下载图标关闭'
            };
            var args = statistics[type].split('|');
            var arg = '//collectdata.gm99.com/ps.gif?id=1&e1=1&e2=gmwancom&e14=' + args[0] + '&e15=' + args[1];
            var temp = document.createElement('div');
            temp.style.position = 'fixed';
            temp.innerHTML = '<img src="' + arg + '">';

            document.getElementsByTagName('body')[0].appendChild(temp);
        },
        // 入口(只在pc端显示下载弹窗)
        bootstrap: function() {
            var self = this;
            self.boxUser(function() {
                self.createDownloadPop();
            }, function() {
                self.hideRightDownloadBt();
            });
        }
    }
    fixedDownloadForGmBox.bootstrap();
    // return fixedDownloadForGmBox;
}));